local is_cata = select(4, _G.GetBuildInfo()) >= 100000;

local frame = CreateFrame("Frame");
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("CHAT_MSG_LOOT");
frame:RegisterEvent("PLAYER_LOGOUT");
frame:RegisterEvent("CURRENCY_DISPLAY_UPDATE");

frame:SetScript("OnEvent", function(self, event, addon) zeldaFrame_OnEvent(self, event, addon); end);


-- Register "anti-sound-spam" feature
sound_played_ts = {};

sound_played_ts[-1] = {
	timestamp = nil,
	interval  = 2
};

sound_played_ts[2] = {
	timestamp = nil,
	interval  = 2
};

sound_played_ts[3] = {
	timestamp = nil,
	interval  = 2
};

sound_played_ts[4] = {
	timestamp = nil,
	interval  = 2
};

sound_played_ts[5] = {
	timestamp = nil,
	interval  = 7
};

sound_played_ts[6] = {
	timestamp = nil,
	interval  = 15
};

-- ------------------ Patch 4.0.6 fix ------------------ --

zle_currencies        = {};
zle_currencies_init   = false;

local ev_ding2        = false;


-- There is no evident link between GetArchaeologyRaceInfo() and GetCurrencyInfo() -_-
function arch_texture_to_currencyID(t)
	if     (t == 'Interface\\Archeology\\Arch-Race-Draenei') then
		return 398;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Dwarf') then
		return 384;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Fossil') then
		return 393;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Nerubian') then
		return 400;

	elseif (t == 'Interface\\Archeology\\Arch-Race-NightElf') then
		return 394;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Orc') then
		return 397;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Tolvir') then
		return 401;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Troll') then
		return 385;

	elseif (t == 'Interface\\Archeology\\Arch-Race-Vrykul') then
		return 399;

	else
		return 1;

	end
end

local pattern_cache = {}
function deformat(str, format)
	local pattern = pattern_cache[format];
	if not pattern then
		-- Escape special characters
		pattern = format:gsub("[%^%$%(%)%%%.%[%]%*%+%-%?]",
			function(c) return "%"..c end);

		-- Substitute formatting elements with captures (only s and d
		-- supported now). Obviously now a formatting element will look
		-- like %%s because we escaped the %.
	    pattern = pattern:gsub("%%%%([sd])", {
			["s"] = "(.-)",
			["d"] = "(%d+)",
		});

		pattern_cache[format] = pattern;
	end

	return str:match(pattern);
end


-- ------------------ /Patch 4.0.6 fix ------------------ --

function play_zeldaSound(set_id, index)
	local sound_set;
	local set_idx;

	local q    = {nil, "green", "blue", "purple", "orange"};
	local idx2 = index;

	if (set_id == nil) then
		-- Automatic from config
		set_idx = zle_config["sounds"]["sets"][q[index]];
	else
		-- Manually set
		set_idx = set_id;
	end

	if (set_idx == 0) then
		sound_set = "ALTTP";

	elseif (set_idx == 1) then
		sound_set = "OOT";

	elseif (set_idx == 2) then
		sound_set = "TP";

	elseif (set_idx == 3) then
		sound_set = "TP";
		
	elseif (set_idx == 4) then
		sound_set = "BOTW";
		
	elseif (set_idx == 5) then
		sound_set = "ALBW";	
		
	elseif (set_idx == 6) then
		sound_set = "LAR";	
	
	elseif (set_idx == 7) then
		sound_set = "LA";
		
	elseif (set_idx == 8) then
		sound_set = "WW";

	elseif (set_idx == 9) then
		sound_set = "SS";
		index = 6;
	
	else
		sound_set = "ALTTP";

	end


	-- Check if multiple sounds will play at once or quite so.
	-- If so, only play one per each quality level.
	--
	-- Leave a few seconds beween each,
	-- but different sounds can ring at once.

	if (zle_config.use_antispam) then
		if (sound_played_ts[idx2].timestamp ~= nil) then
			--print(time(), sound_played_ts[idx2].timestamp);
			if (time() - sound_played_ts[idx2].timestamp < sound_played_ts[index].interval) then
				return;
			end
		end

		sound_played_ts[idx2].timestamp = time();
	end

	-- Fetch sound file and play it

	local f = "[nil]";

	if     (index == 6)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\"..sound_set.."\\05 - Orange - Legendary objects (2).ogg";

	elseif (index == 5)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\"..sound_set.."\\04 - Orange - Legendary objects.ogg";

	elseif (index == 4)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\"..sound_set.."\\03 - Purple - Epic objects.ogg";

	elseif (index == 3)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\"..sound_set.."\\02 - Blue - Rare objects.ogg";

	elseif (index == 2)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\"..sound_set.."\\01 - Green - Not common objects.ogg";

	elseif (index == -1)
		then f = "Interface\\AddOns\\ZeldaLoot_Extended\\Sounds\\Sets\\ALTTP\\05 - Archeological Fragment.ogg";

	else
		return;

	end

	-- Patch 4.0.6 update
	-- Use Master channel for sounds, if available.
	if (is_cata) then
		PlaySoundFile(f, "Master");
	else
		PlaySoundFile(f);
	end
end

function zle_currency_info(id)
	local result;

	if (id == nil) then
		result = {
			text = "",
			amount = 0,
			icon = ""
		};
	else
		local info = C_CurrencyInfo.GetCurrencyInfo(id);
		result = {
			text   = info.name,
			amount = info.quantity,
			icon   = info.iconFileID
		}
	end

	return result;
end

currency_idx = 1;

function zeldaFrame_OnEvent(self, event, ...)
	local obj;

	local iCat, vCat;
	local iSub, vSub;

	local scanCateg = {"green", "blue", "purple", "orange"};
	local scanValues = {active = "loot", crafted = "crafts", received = "received"};

	local qualities_dic = {nil, nil, "green", "blue", "purple", "orange"};
	local quality, zle_group;

	local arg1 = select(1, ...);

	if (((event == "ADDON_LOADED") and (arg1 == "ZeldaLoot_Extended")) or (event == "PLAYER_LOGOUT")) then

		if ((zle_config == nil) or (zle_config_temp == nil) or (zle_config.version == nil) or (zle_config.version < 2.07)) then
			-- First initialization
			zle_config = {
				green = {
					active   = true,
					received = false,
					crafted  = false
				},

				blue = {
					active   = true,
					received = false,
					crafted  = false
				},

				purple = {
					active   = true,
					received = false,
					crafted  = false
				},

				orange = {
					active   = true,
					received = false,
					crafted  = false
				},

				inherited = {
					include  = true
				},

				sounds = {
					sets = {
						green  = 0,
						blue   = 0,
						purple = 0,
						orange = 0
					}
				},

				version = 2.07
			};
		end

		if (zle_config.version < 2.07) then
			zle_config.version = 2.07;
		end

		if (zle_config.currency == nil) then
			zle_config.currency = {
				jp = {
					active = true
				},

				hp = {
					active = true
				},

				misc = {
					active = false
				}
			};
		end

		if (zle_config.sounds.sets.currency == nil) then
			zle_config.sounds.sets.currency = {
				jp = 0,
				hp = 0,
				misc = 0
			};
		end

		if (zle_config.sounds.sets.archaeology == nil) then
			zle_config.sounds.sets.archaeology = 0;
		end

		if (zle_config.archaeology == nil) then
			zle_config.archaeology = {
				active = true
			};
		end

		if (zle_config.use_antispam == nil) then
			zle_config.use_antispam = true;
		end

		zle_config_temp = zle_config;

		for index=1, C_CurrencyInfo.GetCurrencyListSize() do
			--local name, isHeader, isExpanded, isUnused, isWatched, count, icon, maximum, hasWeeklyLimit, currentWeeklyAmount, unknown = GetCurrencyListInfo(index);
			local info = C_CurrencyInfo.GetCurrencyListInfo(index);

			if (not info.isHeader) then
				zle_currencies[info.name] = {
					text   = info.name,
					amount = info.quantity,
					icon   = info.iconFileID
				};
			end
		end

		if (GetNumArchaeologyRaces() > 0) then
			for index=1, GetNumArchaeologyRaces() do
				-- raceName, raceCurrency, raceTexture, raceItemID = GetArchaeologyRaceInfo(index);
				-- 4.0.6 fix
				raceName, raceTexture, raceItemID, raceCurrency = GetArchaeologyRaceInfo(index);
				currencyID = arch_texture_to_currencyID(raceTexture);
				local info = C_CurrencyInfo.GetCurrencyInfo(currencyID)

				if (raceCurrency > 0) then
					zle_currencies[info.name] = {
						text   = info.name,
						amount = info.quantity,
						icon   = info.iconFileID
					};
				end
			end
		end

		if (event ~= "PLAYER_LOGOUT") then
			-- Initiate UI v2
			zle_createConfigPanel();
		end
	end

	if (event == "CURRENCY_DISPLAY_UPDATE") then

		local c0, ev_standard_currency;
		local tmp, set_id, idx, ev_ding;

		ev_standard_currency = false;
		ev_ding = false;

		for index=1, C_CurrencyInfo.GetCurrencyListSize() do
			local info = C_CurrencyInfo.GetCurrencyListInfo(index);
			local name, isHeader, count, icon = info.name, info.isHeader, info.quantity, info.iconFileID

			if (isHeader == false) then
				if (((name == ZLE_CFG_JP) or (name == ZLE_CFG_VP)) and zle_config.currency.jp.active) then
					c0 = (zle_currencies[name] or nil);
					if (c0 == nil) then
						zle_currencies[name] = {
							text   = name,
							amount = count,
							icon   = icon
						};
					end

					if (((c0 == nil) or (count > zle_currencies[name].amount)) and zle_currencies_init) then
						tmp = zle_config.sounds.sets.currency.jp;
						set_id = math.floor(tmp / 5);
						idx = (tmp % 5) + 3;

						play_zeldaSound(set_id, idx);
						ev_standard_currency = true;
					end
				elseif (((name == ZLE_CFG_HP) or (name == ZLE_CFG_CP)) and zle_config.currency.hp.active) then
					c0 = (zle_currencies[name] or nil);
					if (c0 == nil) then
						zle_currencies[name] = {
							text   = name,
							amount = count,
							icon   = icon
						};
					end

					if (((c0 == nil) or (count > zle_currencies[name].amount)) and zle_currencies_init) then
						tmp = zle_config.sounds.sets.currency.hp;
						set_id = math.floor(tmp / 5);
						idx = (tmp % 5) + 3;

						play_zeldaSound(set_id, idx);
						ev_standard_currency = true;
					end
				elseif (zle_config.currency.misc.active) then
					c0 = (zle_currencies[name] or nil);
					if (c0 == nil) then
						zle_currencies[name] = {
							text   = name,
							amount = count,
							icon   = icon
						};
					end

					if (((c0 == nil) or (count > zle_currencies[name].amount)) and zle_currencies_init) then
						tmp = zle_config.sounds.sets.currency.misc;
						set_id = math.floor(tmp / 4);
						idx = (tmp % 4) + 2;

						play_zeldaSound(set_id, idx);
						ev_standard_currency = true;
					end
				end

				if (name ~= nil) then
					zle_currencies[name].amount = count;
				end

				if (ev_standard_currency) then
					break;
				end
			end
		end

		if ((GetNumArchaeologyRaces() > 0) and (ev_standard_currency == false) and (zle_config.archaeology.active)) then

			for index=1, GetNumArchaeologyRaces() do
				--raceName, raceCurrency, raceTexture, raceItemID = GetArchaeologyRaceInfo(index);
				-- 4.0.6 fix
				raceName, raceTexture, raceItemID, raceCurrency = GetArchaeologyRaceInfo(index);
				currencyID = arch_texture_to_currencyID(raceTexture);
				local info = C_CurrencyInfo.GetCurrencyInfo(currencyID);


				c0 = (zle_currencies[localized_label] or nil);
				if (raceCurrency > 0) then
					if (c0 == nil) then
						zle_currencies[info.name] = {
							text   = info.name,
							amount = info.quantity,
							icon   = info.iconFileID
						};

						ev_ding = true;

					elseif ((count > zle_currencies[info.name].amount) or (ev_ding2)) then
						if (ev_ding2) then
							ev_ding2 = false;
							ev_ding  = false;
						else
							ev_ding  = true;
							ev_ding2 = false;
						end

					elseif (count < zle_currencies[info.name].amount) then
						ev_ding  = false;
						ev_ding2 = true;
					end

					zle_currencies[info.name].amount = count;
				end

				if (ev_ding) then
					play_zeldaSound(nil, -1);
					break;
				end
			end
		end

		zle_currencies_init = true;
	end

	if (event == "CHAT_MSG_LOOT") then

		-- Inherited stuff ("artefacts", bind to account - not archaeology crafts !)
		if     (strfind(arg1, "cffe6cc80")) then
			if (zle_config["inherited"]["include"]) then
				quality = 6;
			else
				quality = nil;
			end

		-- Legendary (orange)
		elseif (strfind(arg1, "cffff8000")) then
			quality = 6;

		-- Epic (purple)
		elseif (strfind(arg1, "cffa335ee")) then
			quality = 5;

		-- Rare (blue)
		elseif (strfind(arg1, "cff0070dd")) then
			quality = 4;

		-- Not common (green)
		elseif (strfind(arg1, "cff1eff00")) then
			quality = 3;

		-- Trash: common and low quality (white and grey), no sound for those ones !
		else
			quality = nil;
		end

		local l1 = deformat(arg1, LOOT_ITEM_SELF);
		local l2 = deformat(arg1, LOOT_ITEM_SELF_MULTIPLE);

		local c1 = deformat(arg1, LOOT_ITEM_CREATED_SELF);
		local c2 = deformat(arg1, LOOT_ITEM_CREATED_SELF_MULTIPLE);

		local p1 = deformat(arg1, LOOT_ITEM_PUSHED_SELF);
		local p2 = deformat(arg1, LOOT_ITEM_PUSHED_SELF_MULTIPLE);

		-- As archaeology crafts are mainly trashs (grey objects),
		-- a currencies/fragments rescan is required.
		if (c1 or c2) then
			-- Patch 4.1
			-- Nth+1 try to fix sounds for archaeology after an item is crafted.
			for index=1, C_CurrencyInfo.GetCurrencyListSize() do
				--local name, isHeader, isExpanded, isUnused, isWatched, count, icon, maximum, hasWeeklyLimit, currentWeeklyAmount, unknown = GetCurrencyListInfo(index);
				local info = C_CurrencyInfo.GetCurrencyListInfo(index);

				if (not info.isHeader) then
					zle_currencies[info.name] = {
						text   = info.name,
						amount = info.quantity,
						icon   = info.iconFileID
					};
				end
			end

			if (GetNumArchaeologyRaces() > 0) then
				for index=1, GetNumArchaeologyRaces() do
					-- raceName, raceCurrency, raceTexture, raceItemID = GetArchaeologyRaceInfo(index);
					-- 4.0.6 fix
					raceName, raceTexture, raceItemID, raceCurrency = GetArchaeologyRaceInfo(index);
					currencyID = arch_texture_to_currencyID(raceTexture);
					local info = GetCurrencyInfo(currencyID);

					if (raceCurrency > 0) then
						zle_currencies[info.name] = {
							text   = info.name,
							amount = info.quantity,
							icon   = info.iconFileID
						};
					end
				end
			end
		end

		if (quality) then
			zle_group = qualities_dic[quality];
			if (arg1 == nil) then
				arg1 = "";
			end

			-- This new routine is now locale independant, making
			-- old ZLE_LOOTMESSAGE and other constants obsolete.
			if (zle_config[zle_group]["active"]) then

				if ((c1 or c2) and zle_config[zle_group]["crafted"]) then
					play_zeldaSound(nil, quality - 1);

				elseif ((p1 or p2) and zle_config[zle_group]["received"]) then
					play_zeldaSound(nil, quality - 1);

				elseif (l1 or l2) then
					play_zeldaSound(nil, quality - 1);
				end

			end
		end
	end
end